#! /usr/bin/env python

# faststack.py --- a stack implementation for use in Python programs
# Copyright (c) 2001 Mark Lutz, Florent Rougon
# Copyright (c) 2004 Florent Rougon
#
# This file is part of TDBSF.
#
# This file is (very) heavily inspired from an example in Mark Lutz's
# "Programming Python" (1st edition) book from O'Reilly.
#
# TDBSF is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# TDBSF is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
# License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with TDBSF; see the file COPYING. If not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from __future__ import nested_scopes
import string


class FastStack:
    def __init__(self, start=[]):              # init from any sequence
        self.stack = None                      # even other (fast)stacks
        for i in range(-len(start), 0): 
            self.push(start[-i - 1])           # push in reverse order

    def push(self, node):                      # grow tree 'up/left'
        self.stack = node, self.stack          # new root tuple: (node, tree)

    def pop(self): 
        node, self.stack = self.stack          # remove root tuple
        return node                            # TypeError if empty

    def empty(self): 
        return not self.stack                  # is it 'None'?

    def __len__(self):                         # on: len, not
        len, tree = 0, self.stack
        while tree:
            len, tree = len+1, tree[1]         # visit right subtrees
        return len

    def __getitem__(self, index):              # on: x[i], in, for
        len, tree = 0, self.stack
        while len < index and tree:            # visit/count nodes
            len, tree = len+1, tree[1] 
        if tree:
            return tree[0]                     # IndexError if out-of-bounds
        else: raise IndexError                 # so 'in' and 'for' stop

    def __repr__(self): return '[FastStack:' + `self.stack` + ']'

    def pretty_print(self):
        retval = [ "<FastStack:" ]
        tree = self.stack
        while tree:
            retval.append(" '" + str(tree[0].nom) + "'")
            tree = tree[1]
            if tree:
                retval.append(",")
        retval.append(" >")
        return string.join(retval, '')
